﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Collections.ObjectModel;

namespace OptimisationManagerExtention.Model
{
    /// <summary>
    /// Интерфейс модели
    /// </summary>
    interface IExtentionGUI_M : INotifyPropertyChanged
    {
        #region Properties

        /// <summary>
        /// Устанавливает когда доступна для редактирования таблица с выбранными терминаллами а когда нет
        /// </summary>
        bool IsTerminalsLVEnabled { get; }

        /// <summary>
        /// Список параметров робота как в таблице перед началом оптимизации
        /// </summary>
        List<FileReaders.ParamsItem> BotParams { get; }

        /// <summary>
        /// Статус выполняемой операции
        /// </summary>
        VarKeeper<string> Status { get; }
        /// <summary>
        /// Значение прогресс бара
        /// </summary>
        VarKeeper<double> PB_Value { get; }

        /// <summary>
        /// ID терминалов на стартавой странице
        /// определяются при старте программы и неизменны до ее завершения 
        /// </summary>
        ObservableCollection<string> TerminalsID { get; }

        /// <summary>
        /// Таблица с результатами оптимизации на оптимизируемом отрезке времени
        /// </summary>
        DataTable HistoryOptimisationResults { get; }
        /// <summary>
        /// Таблица с результатами оптимизаций на форвардном отрезке времени
        /// </summary>
        DataTable ForvardOptimisationResults { get; }
        /// <summary>
        /// Коллекция с заголовками отображаемых колонок в таблицах с результатами оптимизаций
        /// </summary>
        ObservableCollection<ViewExtention.ColumnDescriptor> OptimisationResultsColumnHeadders { get; }
        /// <summary>
        /// Список ID терминаллов на которых производилась оптимизация
        /// </summary>
        ObservableCollection<string> TerminalsAfterOptimisation { get; }
        /// <summary>
        /// индекс выбранного терминалла для конкретного робота
        /// </summary>
        VarKeeper<int> TerminalsAfterOptimisation_Selected { get; set; }

        /// <summary>
        /// Список роботов прошедших оптимизацию - роботы в списке не повторяются, а вот терминаллов на одного робота может быть несколько
        /// </summary>
        ObservableCollection<string> BotsAfterOptimisation { get; }
        /// <summary>
        /// индекс выбранного прооптимизированного робота
        /// </summary>
        VarKeeper<int> BotsAfterOptimisation_Selected { get; set; }
        #endregion

        /// <summary>
        /// Метод загружающий историю оптимизаций в таблицы
        /// [History/Forvard]OptimisationResults
        /// </summary>
        /// <param name="terminal_ind">Индекс выбранного робота из списка "BotList"</param>
        /// <param name="bot_ind">Индекс выбранного робота из списка "BotList"</param>
        void LoadOptimisations();
        /// <summary>
        /// Метод проверяющий есть ли сейчас возможность загрузки отчета оптимизации или же нет
        /// </summary>
        /// <returns>true если есть возможность, false если нет</returns>
        //bool IsEnableLoadOptimisationReport();
        /// <summary>
        /// Данный метод вызывается при выборе эксперта для оптимизации / теста 
        /// из таблицы с терминаллами на вкладке Settings
        /// 
        /// Во время его работы :
        /// 1. Сохранятся в прошлом выбранном роботе его оптимизационные настройки - "optimisationInputData"
        /// 2. Загружаются параметры нововыбранного робота в таблицу "BotParams"
        /// 3. в переменную "optimisationInputData" - передается значения настроек оптимизации 
        ///    для нововыбранного робота
        /// </summary>
        /// <param name="fullExpertName">Путь к эксперту относительно папки ~/Experts</param>
        /// <param name="TerminalID">ID запускаемого терминалла</param>
        /// <param name="optimisationInputData">настройки оптимизации</param>
        void LoadBotParams(string fullExpertName,
            string TerminalID,
            out OptimisationInputData? optimisationInputData);
        /// <summary>
        /// Метод получает полные имена роботов для выбранного терминалла относительно папки ~/Experts
        /// </summary>
        /// <param name="terminalIndex">Индекс выбранного терминалла</param>
        /// <returns>
        /// Список полных имен роботов(с учетом пути относительно папки ~/Experts)
        /// </returns>
        List<string> GetBotNamesList(int terminalIndex);
        /// <summary>
        /// Метод получает список доступных активов из выбранного терминала
        /// </summary>
        /// <param name="terminalIndex">индекс выбранного терминалла</param>
        /// <returns>Список доступных активов</returns>
       // List<string> GetAssetNames(int terminalIndex);
        /// <summary>
        /// Метод получает логин из мнмциализационного файла для выбранного терминалла
        /// </summary>
        /// <param name="terminalIndex">индекс выбранного терминалла</param>
        /// <returns>Логин терминалла</returns>
        uint? GetCurrentLogin(int terminalIndex);
        /// <summary>
        /// Проверка перед запуском оптимизации или же теста
        /// </summary>
        /// <param name="SelectedTerminals">список выбранных терминалов для оптимизации/теста</param>
        /// <returns>результат проверки</returns>
        //bool IsEnableToStartOptimisationOrTest(List<ViewModel.TerminalAndBotItem> SelectedTerminals);
        /// <summary>
        /// Метод запускающий тест или же оптимизацию
        /// </summary>
        /// <param name="SelectedTerminals">список выбранных терминалов для оптимизации/теста</param>
        void StartOptimisationOrTest(List<ViewModel.TerminalAndBotItem> SelectedTerminals);
        /// <summary>
        /// Запуск теста по событию двойного клика
        /// </summary>
        /// <param name="TableType">Тип таблицы</param>
        /// <param name="rowIndex">Индекс выбранной натписи</param>
        /// <param name="terminalData">Параметры для входа в терминалл и путь к роботу</param>
        void StartTest(ENUM_TableType TableType, int rowIndex);
        /// <summary>
        /// Вызывается при удалении переданного терминалла из списка терминаллов
        /// В методе удаляются параметры для выбранного робота в данном терминале
        /// </summary>
        /// <param name="item">выбранный для удаления терминалл</param>
        bool RemoveBotParams(string TerminalID);
        /// <summary>
        /// Проверяем можно ли добавить новый терминалл. Нельзя добавлять если в текущий попнт идет оптимизация
        /// </summary>
        /// <returns></returns>
        bool IsEnableToAddNewTerminal();
        void SelectNewBotsAfterOptimisation_forNewTerminal();
        void UpdateTerminalOptimisationsParams(OptimisationInputData optimisationInputData);
        // string GetPathToTerminalFolder();
    }

    #region Accessory objects 

    /// <summary>
    /// Enum характеризующий тип таблиц с результатами оптимизаций
    /// </summary>
    enum ENUM_TableType
    {
        History,
        Forvard
    }

    #endregion
}
