﻿using System;
using System.Collections.Generic;
using System.IO;

namespace OptimisationManagerExtention.Model.Terminal
{
    /// <summary>
    /// Интерфейс менеджера терминаллов
    /// </summary>
    interface ITerminalManager
    {
        /// <summary>
        /// Login для старта - флаг /Login
        /// </summary>
        uint? Login { get; set; }
        /// <summary>
        /// запуск платформы под определенным профилем. 
        /// Профиль должен быть заранее создан и находится в папке /profiles/charts/ торговой платформы
        /// </summary>
        string Profile { get; set; }
        /// <summary>
        /// Конфиг файл ввиде объекта /Config
        /// </summary>
        Config Config { get; set; }
        /// <summary>
        /// Флаг запуска терминалла в режиме /portable
        /// </summary>
        bool Portable { get; set; }
        /// <summary>
        /// стиль окна запускаемого процесса
        /// </summary>
        System.Diagnostics.ProcessWindowStyle WindowStyle { get; set; }
        /// <summary>
        /// Путь к папке где установлен терминалл
        /// </summary>
        DirectoryInfo TerminalInstallationDirectory { get; }
        /// <summary>
        /// Путь к папке терминалла с изменяемыми файлами
        /// </summary>
        DirectoryInfo TerminalChangeableDirectory { get; }
        /// <summary>
        /// Путь к папке MQL5
        /// </summary>
        DirectoryInfo MQL5Directory { get; }
        /// <summary>
        /// Список полных имен экспертов
        /// </summary>
        List<string> Experts { get; }
        /// <summary>
        /// Список полных имен индикаторов
        /// </summary>
        List<string> Indicators { get; }
        /// <summary>
        /// Список полных имен скриптов
        /// </summary>
        List<string> Scripts { get; }
        /// <summary>
        /// ID терминалла - имя папки вдиректории AppData
        /// </summary>
        string TerminalID { get; }
        /// <summary>
        /// Признат того запущен ли терминалл в данный момент или же нет
        /// </summary>
        bool IsActive { get; }

        /// <summary>
        /// Запуск терминалла
        /// </summary>
        bool Run();
        /// <summary>
        /// Остановка процесса
        /// </summary>
        void Close();
        /// <summary>
        /// Дождаться завержения работы терминалла
        /// </summary>
        void WaitForStop();
        /// <summary>
        /// Дождаться завержения работы терминалла определенное время
        /// </summary>
        bool WaitForStop(int miliseconds);

        /// <summary>
        /// Событие завершения запущенного процесса
        /// </summary>
        event Action<ITerminalManager> TerminalClosed;
    }
}
