﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OptimisationManagerExtention.Model.Terminal;
using System.IO;

namespace OptimisationManagerExtentionTests
{
    [TestClass]
    public class ConfigTests
    {
        private string ConfigName = $"{Environment.CurrentDirectory}\\MyTestConfig.ini";
        private string first_excention = "first getters call mast be null becouse file doesn`t contains this key";
        Config config;

        [TestInitialize]
        public void TestInitialize()
        {
            if (File.Exists(ConfigName))
                File.Delete(ConfigName);
            config = new Config(ConfigName);
        }
        [TestMethod]
        public void StringConverter_GetSetTest()
        {
            string expected = null;

            // first get
            string s = config.Common.Password;
            Assert.AreEqual(expected, s, first_excention);

            // set
            expected = "MyTestPassward";
            config.Common.Password = expected;
            s = config.Common.Password;
            Assert.AreEqual(expected, s, "Login mast be equal to MyTestLogin");

            // set null
            config.Common.Login = null;
            s = config.Common.Password;
            Assert.AreEqual(expected, s, "Login mast be equal to MyTestLogin");
        }
        [TestMethod]
        public void ServerConverter_GetSetTest()
        {
            ServerAddressKeeper expected = null;

            // first get;
            ServerAddressKeeper server = config.Common.Server;
            Assert.AreEqual(expected, server);

            // set
            expected = new ServerAddressKeeper("193.219.127.76:4443"); // Open broker demo server
            config.Common.Server = expected;
            server = config.Common.Server;
            Assert.AreEqual(server.Address, expected.Address, $"Adress mast be {expected.Address}");
        }
        [TestMethod]
        public void BoolConverter_GetSetTest()
        {
            bool? expected = null;

            // first get
            bool? b = config.Common.ProxyEnable;
            Assert.AreEqual(expected, b, first_excention);

            // set
            Random gen = new Random();
            int prob = gen.Next(100);
            expected = prob <= 50;
            config.Common.ProxyEnable = expected;
            b = config.Common.ProxyEnable;
            Assert.AreEqual(expected.Value, b.Value, "ProxyEnables mast be equal to true");

            // set null
            config.Common.ProxyEnable = null;
            b = config.Common.ProxyEnable;
            Assert.AreEqual(expected.Value, b.Value, "ProxyEnables mast be equal to true");

        }
        [TestMethod]
        public void ENUMConverter_GetSetTest()
        {
            ENUM_ProxyType? expected = null;

            // first get
            ENUM_ProxyType? p = config.Common.ProxyType;
            Assert.AreEqual(expected, p, first_excention);

            // set
            Random gen = new Random();
            int prob = gen.Next(300);
            int n = prob <= 100 ? 0 : (prob > 100 && prob <= 200 ? 1 : 2);
            expected = (ENUM_ProxyType)n;

            config.Common.ProxyType = expected;
            p = config.Common.ProxyType;
            Assert.AreEqual(expected.Value, p.Value, $"ProxyType mast be equal to {expected.Value}");

            // set null
            config.Common.ProxyEnable = null;
            p = config.Common.ProxyType;
            Assert.AreEqual(expected.Value, p.Value, $"ProxyType mast be equal to {expected.Value}");
        }
        [TestMethod]
        public void DTConverter_GetSetTest()
        {
            DateTime? expected = null;

            // first get
            DateTime? p = config.Tester.FromDate;
            Assert.AreEqual(expected, p, first_excention);

            // set
            expected = DateTime.Now;

            config.Tester.FromDate = expected;
            p = config.Tester.FromDate;
            Assert.AreEqual(expected.Value.Date, p.Value.Date, $"ProxyType mast be equal to {expected.Value}");

            // set null
            config.Common.ProxyEnable = null;
            p = config.Tester.FromDate;
            Assert.AreEqual(expected.Value.Date, p.Value.Date, $"ProxyType mast be equal to {expected.Value}");
        }
        [TestMethod]
        public void DoubleConverter_GetSetTest()
        {
            double? expected = null;

            // first get
            double? p = config.Tester.Deposit;
            Assert.AreEqual(expected, p, first_excention);

            // set
            Random rnd = new Random();
            expected = rnd.NextDouble();

            config.Tester.Deposit = expected;
            p = config.Tester.Deposit;
            Assert.AreEqual(Math.Round(expected.Value, 6), Math.Round(p.Value, 6), $"Deposit mast be equal to {expected.Value}");

            // set null
            config.Common.ProxyEnable = null;
            p = config.Tester.Deposit;
            Assert.AreEqual(Math.Round(expected.Value, 6), Math.Round(p.Value, 6), $"Deposit mast be equal to {expected.Value}");
        }
        [TestMethod]
        public void DeleteKeyTest()
        {
            config.Common.Login = 12345;
            config.DeleteKey(ENUM_SectionType.Common, "Login");

            Assert.AreEqual(null, config.Common.Login, "Key mast be deleted");
        }
        [TestMethod]
        public void DeleteSectionTest()
        {
            config.Common.Login = 12345;
            config.DeleteSection(ENUM_SectionType.Common);

            Assert.AreEqual(null, config.Common.Login, "Key mast be deleted");
        }
    }
}
