﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using OptimisationManagerExtention.Model;
using OptimisationManagerExtention.Model.FileReaders;
using OptimisationManagerExtention.Model.OptimisationManager;
using OptimisationManagerExtention.Model.Terminal;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OptimisationManagerExtentionTests
{
    [TestClass]
    public class OptimisatorSettingsManagerTest
    {
        private static string path = Environment.CurrentDirectory;

        #region InputData
        private static ConfigCreator_inputData configCreatorData;
        private static OptimisationInputData optData;
        private static List<ParamsItem> Params;
        #endregion

        private readonly Comparer_Settings comparer = new Comparer_Settings();

        [ClassInitialize]
        public static void SetUp(TestContext testContext)
        {
            if (File.Exists(Path.Combine(path, "Settings.xml")))
                File.Delete(Path.Combine(path, "Settings.xml"));

            configCreatorData = new ConfigCreator_inputData
            {
                TerminalID = "D0E8209F77C8CF37AD8BF550E51FF075",
                Login = 152025,
                TF = ENUM_Timeframes.D1,
                From = DateTime.Now.Date.AddYears(-1),
                OptimisationMode = ENUM_OptimisationMode.All_symbols_selected_in_Market_Watch,
                Pass = "qwerty123",
                pathToBot = "Advisors\\ExpertMACD.ex5",
                ReportName = "MyTestReport",
                setFileName = "ExpertMACD.set",
                Symbol = "Si Splice",
                Till = DateTime.Now.Date
            };
            optData = new OptimisationInputData
            {
                Currency = "Rub",
                ForvardDate = DateTime.Now.Date.AddMonths(-1),
                IsVisual = true,
                Deposit = 10000,
                Laverage = "1:1",
                DepositIndex = 0,
                ExecutionDelayIndex = (int)ENUM_ExecutionDelay.ms_1,
                ModelIndex = (int)ENUM_Model.OHLC_1_minute,
                CurrencyIndex = 2,
                LaverageIndex = 0,
                OptimisationCriteriaIndex = (int)ENUM_OptimisationCriteria.Balance
            };

            Params = new List<ParamsItem>
            {
                new ParamsItem("Param 1")
                {
                    IsOptimize = true,
                    Start = "1",
                    Step = "1",
                    Stop = "2",
                    Value = "3",
                },
                new ParamsItem("Param 2")
                {
                    IsOptimize = false,
                    Start = null,
                    Step = null,
                    Stop = null,
                    Value = "MyValue",
                },
                new ParamsItem("Param 3")
                {
                    IsOptimize = false,
                    Start = "2",
                    Step = "10",
                    Stop = "200",
                    Value = "3000",
                }
            };
        }

        [TestMethod]
        public void RWTest_Expected_InputData()
        {
            // write
            using (OptimisatorSettingsManager manager = new OptimisatorSettingsManager(path))
            {
                manager.ConfigCreator_inputData = configCreatorData;
                manager.OptimisationInputData = optData;
                manager.SetFileParams = Params;
            }

            // read
            using (OptimisatorSettingsManager manager = new OptimisatorSettingsManager(path))
            {
                var _configCreatorData = manager.ConfigCreator_inputData;
                var _optData = manager.OptimisationInputData;
                var _Params = manager.SetFileParams;

                CollectionAssert.AreEqual(Params, _Params, comparer);
                Assert.AreEqual(true, comparer.Compare(_optData, optData));
                Assert.AreEqual(true, comparer.Compare(_configCreatorData, configCreatorData));
            }
        }
    }

    class Comparer_Settings : System.Collections.IComparer
    {
        public int Compare(object x, object y)
        {
            ParamsItem _x = (ParamsItem)x;
            ParamsItem _y = (ParamsItem)y;

            string X = $"{_x.Variable}={_x.Value}||{_x.Start}||{_x.Step}||{_x.Stop}||{(_x.IsOptimize ? "Y" : "N")}";
            string Y = $"{_y.Variable}={_y.Value}||{_y.Start}||{_y.Step}||{_y.Stop}||{(_y.IsOptimize ? "Y" : "N")}";

            return X.CompareTo(Y);
        }

        public bool Compare(ConfigCreator_inputData x, ConfigCreator_inputData y)
        {
            bool TF = x.TF == y.TF;
            bool Login = x.Login == y.Login;
            bool TeminalID = strCmp(x.TerminalID, y.TerminalID);
            bool pathToBot = strCmp(x.pathToBot, y.pathToBot);
            bool setFileName = strCmp(x.setFileName, y.setFileName);
            bool Pass = strCmp(x.Pass, y.Pass);
            bool CertPass = strCmp(x.CertPass, y.CertPass);
            bool Server = strCmp(x.Server, y.Server);
            bool Symbol = strCmp(x.Symbol, y.Symbol);
            bool reportName = strCmp(x.ReportName, y.ReportName);
            bool From = x.From.CompareTo(y.From) == 0;
            bool Till = x.Till.CompareTo(y.Till) == 0;
            bool OptimisationMode = x.OptimisationMode == y.OptimisationMode;

            return (TF && Login && TeminalID && pathToBot && setFileName &&
                    Pass && CertPass && Server && Symbol && reportName && From &&
                    Till && OptimisationMode);
        }

        private bool strCmp(string x, string y)
        {
            bool x_empty = string.IsNullOrEmpty(x) || string.IsNullOrWhiteSpace(x);
            bool y_empty = string.IsNullOrEmpty(y) || string.IsNullOrWhiteSpace(y);

            return ((x_empty && y_empty) || x.CompareTo(y) == 0);
        }

        public bool Compare(OptimisationInputData x, OptimisationInputData y)
        {
            bool Login = x.Login == y.Login;
            bool ForvardDate = x.ForvardDate.CompareTo(y.ForvardDate) == 0;
            bool IsVisual = x.IsVisual == y.IsVisual;
            bool Deposit = x.Deposit == y.Deposit;
            bool Levarage = strCmp(x.Laverage, y.Laverage);
            bool Currency = strCmp(x.Currency, y.Currency);
            bool DepositIndex = x.DepositIndex == y.DepositIndex;
            bool ExecutionDelayIndex = x.ExecutionDelayIndex == y.ExecutionDelayIndex;
            bool ModelIndex = x.ModelIndex == y.ModelIndex;
            bool CurrencyIndex = x.CurrencyIndex == y.CurrencyIndex;
            bool LevarageIndex = x.LaverageIndex == y.LaverageIndex;
            bool OptimisationCriteriaIndex = x.OptimisationCriteriaIndex == y.OptimisationCriteriaIndex;

            return (Login && ForvardDate && IsVisual && Deposit &&
                    Levarage && Currency && DepositIndex && ExecutionDelayIndex &&
                    ModelIndex && CurrencyIndex && LevarageIndex && OptimisationCriteriaIndex);
        }
    }
}
