﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Text;
using OptimisationManagerExtention.Model.FileReaders;
using System.IO;

namespace OptimisationManagerExtentionTests
{
    [TestClass]
    public class SetFileManagerTest
    {
        private static string ParamsName = $"{Environment.CurrentDirectory}\\Params.set";
        private static string ParamsDuplicateFile = $"{Environment.CurrentDirectory}\\ParamsDuplicate.set";
        private SetFileManager setManager;
        private readonly Comparer_SetFile comparer = new Comparer_SetFile();

        [TestInitialize]
        public void TestSetUp()
        {
            if (File.Exists(ParamsName))
                File.Delete(ParamsName);
            using (var file = File.Create(ParamsName))
            {
                byte[] s = Encoding.UTF8.GetBytes(@"MaximumRisk=0.02||0.02||0.002000||0.200000||N
DecreaseFactor=3||3||0.300000||30.000000||N
MovingPeriod=12||12||1||120||N
MovingShift=6||6||1||60||N");
                file.Write(s, 0, s.Length);
            }
            setManager = new SetFileManager(ParamsName, false);
        }
        [ClassCleanup]
        public static void CleadUp()
        {
            File.Delete(ParamsName);
            File.Delete(ParamsDuplicateFile);
        }

        [TestMethod]
        public void CreateNewEmptyFile_Test()
        {
            bool expected = true;

            new SetFileManager(ParamsDuplicateFile, true);
            bool isFileExists = File.Exists(ParamsDuplicateFile);
            File.Delete(ParamsDuplicateFile);

            Assert.AreEqual(expected, isFileExists);
        }
        [TestMethod]
        public void ParamsGetter_FirstLoad_Test()
        {
            List<ParamsItem> expected = GetExpectedParams(false);

            List<ParamsItem> _params = setManager.Params;

            CollectionAssert.AreEqual(expected, _params, comparer);
        }
        [TestMethod]
        public void SaveParams_ChangeIsOptimise_Was_False_expected_True_Test()
        {
            List<ParamsItem> expected = GetExpectedParams(true);

            List<ParamsItem> _params = setManager.Params;
            for (int i = 0; i < _params.Count; i++)
            {
                _params[i].IsOptimize = true;
            }
            setManager.SaveParams();
            _params.Clear(); // Отчищаем что бы параметры заново загрузились именно с файла а не с пмяти
            _params = setManager.Params;

            CollectionAssert.AreEqual(expected, _params, comparer);
        }
        [TestMethod]
        public void DuplicateFile_Test()
        {
            List<ParamsItem> expected = setManager.Params;

            if (File.Exists(ParamsDuplicateFile))
                File.Delete(ParamsDuplicateFile);
            SetFileManager duplicate = setManager.DublicateFile(ParamsDuplicateFile);
            bool isFileExists = File.Exists(ParamsDuplicateFile);
            List<ParamsItem> _params = duplicate.Params;

            Assert.AreEqual(true, isFileExists, "File wasn`t creared!");
            CollectionAssert.AreEqual(expected, _params, comparer);
        }

        private List<ParamsItem> GetExpectedParams(bool isOptimise)
        {
            List<ParamsItem> expected = new List<ParamsItem>
            {
                new ParamsItem("MaximumRisk")
                {
                    IsOptimize = isOptimise,
                    Value = "0.02",
                    Start = "0.02",
                    Step = "0.002000",
                    Stop = "0.200000"
                },
                new ParamsItem("DecreaseFactor")
                {
                    IsOptimize = isOptimise,
                    Value = "3",
                    Start = "3",
                    Step = "0.300000",
                    Stop = "30.000000"
                },
                new ParamsItem("MovingPeriod")
                {
                    IsOptimize = isOptimise,
                    Value = "12",
                    Start = "12",
                    Step = "1",
                    Stop = "120"
                },
                new ParamsItem("MovingShift")
                {
                    IsOptimize = isOptimise,
                    Value = "6",
                    Start = "6",
                    Step = "1",
                    Stop = "60"
                }
            };

            return expected;
        }
    }

    class Comparer_SetFile : System.Collections.IComparer
    {
        public int Compare(object x, object y)
        {
            ParamsItem _x = (ParamsItem)x;
            ParamsItem _y = (ParamsItem)y;

            string __x = $"{_x.Variable}={_x.Value}||{_x.Start}||{_x.Step}||{_x.Stop}||{_x.IsOptimize}";
            string __y = $"{_y.Variable}={_y.Value}||{_y.Start}||{_y.Step}||{_y.Stop}||{_y.IsOptimize}";

            return __x.CompareTo(__y);
        }
    }
}
